import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

class PrefsDialog extends JDialog implements ActionListener, ItemListener {

	private JTextField mwWidth, mwHeight;
	private ButtonGroup fcBG, mwDS, mwTBP, mwLF;
	private JRadioButton fcUseSwing, fcUseNative, mwOutline, mwLive,
	                      top, left, right, bottom, mwJava, mwNative;
	private JCheckBox previewImages, fullScreen, useFileTypeIcons,
	                   useFileFilters;
	private JButton browse, ok, cancel, apply;
	private JFileChooser fileChooser;
	private JTextField dirField;

	private boolean fileChooserCreated;
	private boolean dialogVisible;

	private Properties properties;
	private ImageViewer parent;


	// constants
	static final String okS = "ok";
	static final String cancelS = "cancel";
	static final String browseS = "browse";

	static final String trueS = "true";
	static final String falseS = "false";

	public PrefsDialog(ImageViewer _parent, Properties prop) {

		super(_parent, "Preferences");
		parent = _parent;
		properties = prop;
		fileChooserCreated = false;
		dialogVisible = false;
		buildGUI();
		pack();
		int width = 380;
		int height = 365;
		setSize(width, height);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		Rectangle rect = new Rectangle(new Point(0,0), d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

	}

	private void buildGUI() {

		JPanel contentPane = new JPanel();
		contentPane.setLayout( new BorderLayout() );

		// need to use properties to set values

		JPanel fileChooserPanel = buildFileChooserPanel();
		JPanel mainWindowPanel = buildMainWindowPanel();

		JTabbedPane tabbedPane = new JTabbedPane();
	        tabbedPane.addTab("File Chooser", null, fileChooserPanel, "Preferences for File Chooser");
	        tabbedPane.addTab("Main Window", null, mainWindowPanel, "Preferences for Main Window");

		JPanel buttonsPanel = new JPanel();
		ok = new JButton("Save");
		ok.addActionListener(this);
		ok.setActionCommand(okS);
		cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		cancel.setActionCommand(cancelS);
		buttonsPanel.add(ok);
		buttonsPanel.add(cancel);

		contentPane.add(tabbedPane, BorderLayout.CENTER);
		contentPane.add(buttonsPanel, BorderLayout.SOUTH);

		setContentPane(contentPane);

		// make ok the default button
		getRootPane().setDefaultButton(ok);

	}

	private JPanel buildFileChooserPanel() {

		JPanel fileChooserPanel = new JPanel();

		fileChooserPanel.setPreferredSize( new Dimension(360, 295) );
		fileChooserPanel.setLayout( new GridLayout(3, 1) );

		JPanel type = new JPanel();
		fcBG = new ButtonGroup();
	        fcUseSwing = new JRadioButton("Java");
	        fcUseNative = new JRadioButton("Native");
	        fcUseSwing.addItemListener(this);
	        fcUseNative.addItemListener(this);
	        fcBG.add(fcUseSwing);
	        fcBG.add(fcUseNative);
                type.setBorder( new TitledBorder("Type") );
	        type.add(fcUseSwing);
	        type.add(fcUseNative);

                JPanel optionsPanel = new JPanel();
                optionsPanel.setBorder( new TitledBorder("Options") );
                previewImages = new JCheckBox("Preview Images?");
	        useFileTypeIcons = new JCheckBox("Use File Type Icons?");
	        useFileFilters = new JCheckBox("Use File Filters?");
		optionsPanel.add(previewImages);
                optionsPanel.add(useFileTypeIcons);
                optionsPanel.add(useFileFilters);

		JPanel imageDirPanel = new JPanel();
		imageDirPanel.setBorder( new TitledBorder("Image Directory") );
		dirField = new JTextField(12);
		browse = new JButton("Browse...");
		browse.addActionListener(this);
		browse.setActionCommand(browseS);
		imageDirPanel.add(dirField);
		imageDirPanel.add(browse);

		fileChooserPanel.add(type);
		fileChooserPanel.add(optionsPanel);
		fileChooserPanel.add(imageDirPanel);

		return fileChooserPanel;

	}

	private JPanel buildMainWindowPanel() {

		JPanel mainWindowPanel = new JPanel();
		mainWindowPanel.setPreferredSize( new Dimension(360, 295) );
		mainWindowPanel.setLayout( new GridLayout(4, 1) );

		JPanel sizePanel = new JPanel();
		sizePanel.setBorder( new TitledBorder("Size") );
                sizePanel.add( new JLabel("Width") );
                mwWidth = new JTextField(3);
                sizePanel.add(mwWidth);
                sizePanel.add( new JLabel("   Height") );
                mwHeight = new JTextField(3);
                sizePanel.add(mwHeight);
                sizePanel.add( new JLabel("   ") );
                fullScreen = new JCheckBox("Full Screen?");
                fullScreen.addItemListener(this);
                sizePanel.add(fullScreen);

                JPanel toolBarPanel = new JPanel();
                toolBarPanel.setBorder( new TitledBorder("Tool Bar Position") );
                mwTBP = new ButtonGroup();
                top = new JRadioButton("Top");
                left = new JRadioButton("Left");
                right = new JRadioButton("Right");
                bottom = new JRadioButton("Bottom");
                top.addItemListener(this);
                left.addItemListener(this);
                right.addItemListener(this);
                bottom.addItemListener(this);
                mwTBP.add(top);
                mwTBP.add(left);
                mwTBP.add(right);
                mwTBP.add(bottom);
                toolBarPanel.add(top);
                toolBarPanel.add(left);
                toolBarPanel.add(right);
                toolBarPanel.add(bottom);

		JPanel dragPanel = new JPanel();
                dragPanel.setBorder( new TitledBorder("Drag Style") );
                mwDS = new ButtonGroup();
                mwOutline = new JRadioButton("Outline");
                mwLive = new JRadioButton("Live");
                mwDS.add(mwOutline);
                mwDS.add(mwLive);
                dragPanel.add(mwOutline);
                dragPanel.add(mwLive);

                JPanel lfPanel = new JPanel();
                lfPanel.setBorder( new TitledBorder("Look and Feel") );
                mwLF = new ButtonGroup();
                mwJava = new JRadioButton("Java");
                mwNative = new JRadioButton("Native");
                mwJava.addItemListener(this);
                mwNative.addItemListener(this);
                mwLF.add(mwJava);
                mwLF.add(mwNative);
                lfPanel.add(mwJava);
                lfPanel.add(mwNative);

                mainWindowPanel.add(sizePanel);
                mainWindowPanel.add(toolBarPanel);
                mainWindowPanel.add(dragPanel);
                mainWindowPanel.add(lfPanel);

		return mainWindowPanel;

	}

	public void itemStateChanged(ItemEvent e) {

		Object item = e.getItem();
		int state = e.getStateChange();

		if (item == fcUseSwing) {
		   if (state == ItemEvent.DESELECTED)
		      enableFCOptions(false);
		   else
		      enableFCOptions(true);
		}
                else if (item == fcUseNative) {
                   if (state == ItemEvent.DESELECTED)
                      enableFCOptions(true);
		   else
		      enableFCOptions(false);
		}
		else if (item == fullScreen) {
                   if (state == ItemEvent.DESELECTED)
                      enableMWSize(true);
		   else
		      enableMWSize(false);
		}
		else if ( (item == mwJava) || (item == mwNative) ) {
		   if ( (state == ItemEvent.SELECTED) && dialogVisible )
                      JOptionPane.showMessageDialog(this, "The Look and Feel change will not occur until\n" +
                                                     "JImageView is restarted.",
	                                              "Message", JOptionPane.INFORMATION_MESSAGE);
	        }
	        else if ( (item == top) || (item == left) || (item == bottom) || (item == right) ) {
	            if ( (state == ItemEvent.SELECTED) && dialogVisible )
                      JOptionPane.showMessageDialog(this, "The Tool Bar Position change will not occur until\n" +
                                                     "JImageView is restarted.",
	                                              "Message", JOptionPane.INFORMATION_MESSAGE);
                }
	}

	private void enableFCOptions(boolean tf) {

		previewImages.setEnabled(tf);
		useFileTypeIcons.setEnabled(tf);
	        useFileFilters.setEnabled(tf);

	}

	private void enableMWSize(boolean tf) {

		mwWidth.setEnabled(tf);
		mwHeight.setEnabled(tf);

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(okS) )
		   okAction();
		else if ( actionCommand.equals(cancelS) )
		   setVisible(false);
		else if ( actionCommand.equals(browseS) )
		   browseAction();

	}

	private void okAction() {

		boolean noErrors;

	        // get preferences from components
	        noErrors = getPrefs();

                if (noErrors) {
	           setVisible(false);
                   saveProperties(); // save to config file

                   // reset all things affected by properties (except L&F)
                   //  (filechooser, screen size, toolBar position, drag mode)
                   parent.resetFileChooser();

                   int width = 730;
                   int height = 530;
                   Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
                   if ( fullScreen.isSelected() ) {
                      width = (int)d.getWidth();
	              height = (int)d.getHeight();
		   }
                   else {
                      try {
                           width = Integer.parseInt( mwWidth.getText() );
                           height = Integer.parseInt( mwHeight.getText() );
                      }
                      catch (NumberFormatException nfe) {}
                   }
                   parent.setSize(width, height);
                   Rectangle rect = new Rectangle(new Point(0,0), d);
                   parent.setLocation( (rect.x + ((rect.width - width)/2)),
		                        (rect.y + ((rect.height - height)/2)) );

		   String dragMode;
		   if ( mwOutline.isSelected() )
		      dragMode = PropertiesConstants.outlineS;
		   else
		      dragMode = PropertiesConstants.liveS;
                   parent.resetDragMode(dragMode);

                }

	}

	private boolean getPrefs() {

		String prop = null;
		boolean noErrors;

		noErrors = getFCPrefs(prop);
		if (noErrors)
		   noErrors = getMWPrefs(prop);

		return noErrors;

	}

	private boolean getFCPrefs(String prop) {

	        if ( fcUseSwing.isSelected() )
	           prop = trueS;
	        else
	           prop = falseS;
	        properties.setProperty(PropertiesConstants.useSwingFileChooserS, prop);

	        if ( previewImages.isSelected() )
	           prop = trueS;
	        else
	           prop = falseS;
	        properties.setProperty(PropertiesConstants.previewImagesS, prop);

	        if ( useFileTypeIcons.isSelected() )
	           prop = trueS;
	        else
	           prop = falseS;
	        properties.setProperty(PropertiesConstants.useFileTypeIconsS, prop);

	        if ( useFileFilters.isSelected() )
	           prop = trueS;
	        else
	           prop = falseS;
	        properties.setProperty(PropertiesConstants.useFileFiltersS, prop);

	        prop = dirField.getText();
	        if ( !(new File(prop)).isDirectory() ) { // check to see if the image directory is valid
	           JOptionPane.showMessageDialog(this, "Image Directory does not exist.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
	           return false;
	        }
	        else
	           properties.setProperty(PropertiesConstants.imageDirS, prop);

	        return true;

	}

	private boolean getMWPrefs(String prop) {

                if ( fullScreen.isSelected() ) {
                   prop = trueS;
                   properties.setProperty(PropertiesConstants.fullScreenS, prop);
                }
                else {

                   prop = falseS;
                   properties.setProperty(PropertiesConstants.fullScreenS, prop);
                   int w, h;
                   prop = mwWidth.getText();
                   try {
                        w = Integer.parseInt(prop);
                   }
                   catch (NumberFormatException nfe1) {
                        JOptionPane.showMessageDialog(this, "Width is not a valid number.",
	                                               "Error", JOptionPane.ERROR_MESSAGE);
                   	return false;
                   }
                   properties.setProperty(PropertiesConstants.widthS, prop);
                   prop = mwHeight.getText();
                   try {
                        h = Integer.parseInt(prop);
                   }
                   catch (NumberFormatException nfe2) {
                        JOptionPane.showMessageDialog(this, "Height is not a valid number.",
	                                               "Error", JOptionPane.ERROR_MESSAGE);
	                return false;
                   }
                   properties.setProperty(PropertiesConstants.heightS, prop);

                }

                if ( top.isSelected() )
                   prop = PropertiesConstants.topS;
                else if ( bottom.isSelected() )
                   prop = PropertiesConstants.bottomS;
                else if ( left.isSelected() )
                   prop = PropertiesConstants.leftS;
                else // if ( right.isSelected() )
                   prop = PropertiesConstants.rightS;
		properties.setProperty(PropertiesConstants.toolBarPosS, prop);

		if ( mwOutline.isSelected() )
		   prop = PropertiesConstants.outlineS;
		else
		   prop = PropertiesConstants.liveS;
		properties.setProperty(PropertiesConstants.dragModeS, prop);

		if ( mwJava.isSelected() )
		   prop = trueS;
		else
		   prop = falseS;
		properties.setProperty(PropertiesConstants.useJavaLFS, prop);

		return true;

	}

	private void saveProperties() {

		String dir = System.getProperty("user.home");
     		String configFileWPath = dir+"/.jimageviewrc";

		try {
                     FileOutputStream out = new FileOutputStream(configFileWPath);
             	     BufferedOutputStream bos = new BufferedOutputStream(out);
                     properties.store(bos, null);
                     bos.close();
                     out.close();
                }
                catch (IOException ioe) {
                     System.err.println("Properties file could not be saved");
                }

	}

	/**
	 * Opens a new dialog so that the
	 * user can select the default image directory.
	 */
	private void browseAction() {

		if (!fileChooserCreated) {
		   fileChooser = new JFileChooser();
     		   fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		   fileChooser.setDialogTitle("Select the Default Image Directory");
		   fileChooserCreated = true;
                }
		String prop = properties.getProperty(PropertiesConstants.imageDirS);
		fileChooser.setCurrentDirectory( new File( prop ) );

		int returnVal = fileChooser.showDialog(this, "Select");

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                   File file = fileChooser.getSelectedFile();
                   dirField.setText( file.getPath() );
                }

	}

	public void setVisible(boolean visible) {

		if (visible) // set selected values in prefs dialog from properties
                   setValues();
                else
                   dialogVisible = false;
		super.setVisible(visible);

	}

	private void setValues() {

	        String prop = null;
                boolean tf = false;

                setFCTypeValues(prop, tf);
                setFCOptionsValues(prop, tf);
                setFCImageDirValues(prop, tf);

                setMWSizeValues(prop, tf);
                setMWToolBarPosValues(prop, tf);
                setMWDragStyleValues(prop, tf);
                setMWLFValues(prop, tf);

	}

	private void setFCTypeValues(String prop, boolean tf) {

		prop = properties.getProperty(PropertiesConstants.useSwingFileChooserS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf)
                   fcUseSwing.setSelected(true);
                else
                   fcUseNative.setSelected(true);

	}

	private void setFCOptionsValues(String prop, boolean tf) {

		prop = properties.getProperty(PropertiesConstants.previewImagesS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf)
                   previewImages.setSelected(true);
                else
                   previewImages.setSelected(false);
                prop = properties.getProperty(PropertiesConstants.useFileTypeIconsS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf)
                   useFileTypeIcons.setSelected(true);
                else
                   useFileTypeIcons.setSelected(false);
                prop = properties.getProperty(PropertiesConstants.useFileFiltersS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf)
                   useFileFilters.setSelected(true);
                else
                   useFileFilters.setSelected(false);

	}

        private void setFCImageDirValues(String prop, boolean tf) {

        	prop = properties.getProperty(PropertiesConstants.imageDirS);
        	dirField.setText(prop);

        }

        private void setMWSizeValues(String prop, boolean tf) {

        	prop = properties.getProperty(PropertiesConstants.widthS);
        	mwWidth.setText(prop);
                prop = properties.getProperty(PropertiesConstants.heightS);
                mwHeight.setText(prop);
                prop = properties.getProperty(PropertiesConstants.fullScreenS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf) {
                   // disable height and width
                   mwWidth.setEnabled(false);
                   mwHeight.setEnabled(false);
                   fullScreen.setSelected(true);
                }
                else
                   fullScreen.setSelected(false);

        }

        private void setMWToolBarPosValues(String prop, boolean tf) {

        	prop = properties.getProperty(PropertiesConstants.toolBarPosS);
                if ( prop.equals(PropertiesConstants.topS) )
                   top.setSelected(true);
                else if ( prop.equals(PropertiesConstants.bottomS) )
                   bottom.setSelected(true);
                else if ( prop.equals(PropertiesConstants.leftS) )
                   left.setSelected(true);
                else // if (right)
                   right.setSelected(true);

        }

        private void setMWDragStyleValues(String prop, boolean tf) {

        	prop = properties.getProperty(PropertiesConstants.dragModeS);
                if ( prop.equals(PropertiesConstants.liveS) )
        	   mwLive.setSelected(true);
        	else // if ( prop.equals(outlineS) )
        	   mwOutline.setSelected(true);

        }

        private void setMWLFValues(String prop, boolean tf) {

        	prop = properties.getProperty(PropertiesConstants.useJavaLFS);
                tf = Boolean.valueOf(prop).booleanValue();
                if (tf)
        	   mwJava.setSelected(true);
                else
        	  mwNative.setSelected(true);

        	dialogVisible = true;

        }

}
